module GAME_CNTL (G_MEM_IN, CLK, G_MEM_ADDR, G_MEM_OUT, G_MEM_WE, D_MEM_ADDR, D_DATA, D_WE, SWAP, V_SYNC, RST, START, UI_L, UI_R) ;

input [3:0] G_MEM_IN ;
input CLK ;
output [7:0] G_MEM_ADDR ;
output [3:0] G_MEM_OUT ;
output G_MEM_WE ;
output [7:0] D_MEM_ADDR ;
output [2:0] D_DATA ;
output D_WE ;
output SWAP ;
input V_SYNC ;
input RST;
input START;
input UI_L;
input UI_R;

// add your declarations here
reg [7:0] G_MEM_ADDR ;
reg [3:0] G_MEM_OUT ;
reg G_MEM_WE ;
reg [7:0] D_MEM_ADDR ;
reg [2:0] D_DATA ;
reg D_WE ;
reg SWAP ;

reg [2:0] head_move_state;

reg [1:0] global_state,
    write_state,
    move_state,
    tail_del_state;

reg init_state;

reg [7:0]mem_index;

reg [7:0] head_loc,
	  tail_loc;

reg [3:0] head_data;

reg [15:0] move_count;
reg [1:0] food_count;

// add your code here


always @(posedge CLK) begin
  if (!RST) begin
     if (global_state == 0) begin	// setting up game state
       if (init_state == 0) begin	// setting up g_mem
         G_MEM_ADDR <= mem_index;
         G_MEM_OUT <= 0;
         G_MEM_WE <= 1;
         // add the border
         if ((mem_index[3:0] == 0) | (mem_index[3:0] == 15) | (mem_index[7:4] == 0) | (mem_index[7:4] == 9)) begin
           G_MEM_OUT <= 4'b0100;
         end
         // add some food
         else if ((mem_index[3:0] == 4) & (mem_index[7:4] == 4)) begin
           G_MEM_OUT <= 4'b1000;
         end
         // add the snake's head
         else if ((mem_index[3:0] == 1) & (mem_index[7:4] == 1)) begin
           G_MEM_OUT <= 4'b1101;
           head_loc <= mem_index;
         end
         // add its tail
         else if ((mem_index[3:0] == 8) & (mem_index[7:4] == 5)) begin
           G_MEM_OUT <= 4'b1100;
           tail_loc <= mem_index;
         end
         mem_index <= mem_index + 1;
       
         // done initializing g_mem
         if (mem_index == 159) begin
           mem_index <= 0;
           init_state <= 1;
         end
       end
       else begin	// init_state == 1 writing display
         if (write_state == 0) begin	// setup first read
           G_MEM_ADDR <= 0;
           G_MEM_WE <= 0;
           SWAP <= 0;
           write_state <= 1;
           mem_index <= 0;
         end
         else if (write_state == 1) begin	// doing read and write
           G_MEM_ADDR <= mem_index + 1;
           D_MEM_ADDR <= mem_index;
           D_WE <= 1;
           D_DATA <= {1'b0, G_MEM_IN[3:2]};
           mem_index <= mem_index + 1;
          if (mem_index >= 159) begin		// done writing
            mem_index <= 0;
            write_state <= 2;
          end
         end
         else begin	// write_state == 2, gonna do a swap
           D_WE <= 0;
           if (V_SYNC) begin
             SWAP <= 1;
             write_state <= 0;
             init_state <= 0;
             global_state <= 1;
           end
         end
       end
     end
     else if (global_state == 1) begin	// waiting to start
       SWAP <= 0;
       if (START) begin		// user pressed the start button
         global_state <= 2;
       end
     end
     else begin 	// running
       SWAP <= 0;
       if (move_count == 0) begin
         if (move_state == 0) begin	// writing to doing the head
           if (head_move_state == 0) begin
             G_MEM_ADDR <= head_loc;
             G_MEM_WE <= 0;
             head_move_state <= 1;
           end
           else if (head_move_state == 1) begin
             head_data <= G_MEM_IN;
             if (G_MEM_IN[1:0] == 2'b00) begin	// up
               head_loc[7:4] <= head_loc[7:4] - 1;
             end
             else if (G_MEM_IN[1:0] == 2'b01) begin	// right
               head_loc[3:0] <= head_loc[3:0] + 1;
             end
             if (G_MEM_IN[1:0] == 2'b00) begin	// down
               head_loc[7:4] <= head_loc[7:4] + 1;
             end
             else begin		// left
               head_loc[3:0] <= head_loc[3:0] - 1;
             end
                          
             G_MEM_ADDR <= head_loc;
             head_move_state <= 2;
           end
           else if (head_move_state == 2) begin	
             // check if we hit wall or ourselves
             if ((G_MEM_IN[3:2] == 2'b01) | (G_MEM_IN[3:2] == 2'b11)) begin
               head_move_state <= 0;
               move_state <= 0;
               global_state <= 0;
             end
             // or food...
             else if (G_MEM_IN[3:2] == 2'b10) begin
               food_count <= 1;		// this will grow the tail
             end
           end
           else if (head_move_state == 3) begin
             G_MEM_OUT <= head_data;
             G_MEM_ADDR <= head_loc;
             G_MEM_WE <= 1;
             head_move_state <= 4;
           end
           else if (head_move_state == 4) begin
             G_MEM_WE <= 0;
             head_move_state <= 0;
             move_state <= 1;
             move_count <= 32767;
           end
         end
         else if (move_state == 1) begin	// doing the tail
           if (food_count == 0) begin		// snake not getting any longer
             if (tail_del_state == 0) begin	// read in old tail
               G_MEM_WE <= 0;
               G_MEM_ADDR <= tail_loc;
               tail_del_state <= 1;
             end
             else if (tail_del_state == 1) begin	// adjusting tail_loc
               if (G_MEM_IN[1:0] == 2'b00) begin	// up
                 tail_loc[7:4] <= tail_loc[7:4] - 1;
               end
               else if (G_MEM_IN[1:0] == 2'b01) begin	// right
                 tail_loc[3:0] <= tail_loc[3:0] + 1;
               end
               if (G_MEM_IN[1:0] == 2'b00) begin	// down
                 tail_loc[7:4] <= tail_loc[7:4] + 1;
               end
               else begin		// left
                 tail_loc[3:0] <= tail_loc[3:0] - 1;
               end
               // blank space goes where tail was
               G_MEM_ADDR <= tail_loc;
               G_MEM_OUT <= 0;
               G_MEM_WE <= 1;
               tail_del_state <= 2;
             end
             else begin		// clean up after write
               G_MEM_WE <= 0;
               tail_del_state <= 0;
               move_state <= 2;
             end
           end
           else begin		// snake is growing
             move_state <= 2;
             food_count <= food_count - 1;
           end
         end
         else begin	// move_state == 2, writing to disp
           if (write_state == 0) begin	// setup first read
             G_MEM_ADDR <= 0;
             G_MEM_WE <= 0;
             SWAP <= 0;
             write_state <= 1;
             mem_index <= 0;
           end
           else if (write_state == 1) begin	// doing read and write
             G_MEM_ADDR <= mem_index + 1;
             D_MEM_ADDR <= mem_index;
             D_WE <= 1;
             D_DATA <= {1'b0, G_MEM_IN[3:2]};
             mem_index <= mem_index + 1;
            if (mem_index >= 159) begin		// done writing
              mem_index <= 0;
              write_state <= 2;
            end
           end
           else begin	// write_state == 2, gonna do a swap
             D_WE <= 0;
             if (V_SYNC) begin
               SWAP <= 1;
               write_state <= 0;
               move_state <= 0;
             end
           end
         end
       end
       else begin
         move_count <= move_count - 1;
       end
     end
  end
  else begin
    D_WE <= 0;
    SWAP <= 0;
    G_MEM_WE <= 0;
    mem_index <= 0;
    global_state <= 0;
    init_state <= 0;
    write_state <= 0;
    move_count <= 0;
    move_state <= 0;
  end
end

endmodule
