module SpriteMove (clk, inBus, stroked, Xs, Ys, Xt, Yt) ;


wire right ; // X++
wire left ; // X--
wire up ; // Y++
wire down ; // Y--
wire m1, m2, m3, m4;
input clk;
input [7:0] inBus ;
input stroked;
input [9:0] Xt, Yt;
output [9:0] Xs ;
output [9:0] Ys ;

reg [7:0] inBus;
reg [31:0] counter;
reg [31:0] gravitycounter;
reg [9:0] Xs, Ys;
integer Xvec, Yvec;
reg [2:0] mode;
reg startup;
reg chasing;
parameter still=0, bounce=1, pool=2, chase=4;
parameter UPDATE_FREQUENCY = 1000000;
parameter go_still=4'b1000, go_pool=4'b0100, go_bounce=4'b0010, go_chase=4'b0001;
parameter XMIN = 20, XMAX = 587, YMIN = 20, YMAX = 427;
parameter DAMPER = 5;
parameter GRAVITY = 10'b0000000001;

assign down = (inBus == 8'b01110101) && stroked;
assign up = (inBus == 8'b01110010) && stroked;
assign left = (inBus == 8'b01101011) && stroked;
assign right = (inBus == 8'b01110100) && stroked;
assign m1 =    (inBus == 8'b00011010) && stroked;
assign m2 =    (inBus == 8'b00100010) && stroked;
assign m3 =    (inBus == 8'b00100001) && stroked;
assign m4 =    (inBus == 8'b00101010) && stroked;


always @(posedge clk)
begin
	if(!startup)
	begin
		Xs<=20;
		Ys<=20;	
		startup = 1;
		mode<=still;
	end
	
	if (counter > UPDATE_FREQUENCY)
		counter <= 0;
	else
		counter <= counter + 1;
	
		
	case({m1,m2,m3,m4})
		go_still:
		      begin
			mode <= still;
			Xvec <= 0;
			Yvec <= 0;
		      end
		go_pool:
			mode <= pool;
		go_bounce:
			mode <= bounce;
		go_chase:
			chasing <= !chasing;
		4'b0000:
			mode <= mode;
	       default:
	       	      begin
			mode <= still;
			Xvec <= 0;
			Yvec <= 0;
		      end
	endcase
	
	case(mode)
		still:
		      begin
			if (Xs + right - left > XMIN && Xs + right - left < XMAX)
				Xs <= Xs + 3*(right - left);
			if (Ys + up - down > YMIN && Ys + up - down < YMAX)
				Ys <= Ys + 3*(up - down);
		      end
		pool: 
		      begin
		      	if(counter == 0)
			begin
				if(Xs + Xvec > XMIN && Xs + Xvec < XMAX)
					Xs <= Xs + Xvec;
				else
					Xvec <= -Xvec;
					
				if(Ys + Yvec > YMIN && Ys + Yvec < YMAX)
					Ys <= Ys + Yvec;
				else
					Yvec <= -Yvec;
			end
			else
			begin
				Xvec <= Xvec + right - left;
				Yvec <= Yvec + up - down;
			end
		      end
		bounce: // NOTE: The "ground" is at YMAX, not YMIN.
		      begin
		      	if(counter == 0)
		      	begin
		      		gravitycounter <= gravitycounter + 1;
		      		if(gravitycounter == 10)
		      			gravitycounter <= 0;
				
				if(gravitycounter == 0)
						Yvec <= Yvec + GRAVITY;
					
				if(Xs + Xvec > XMIN && Xs + Xvec < XMAX)
					Xs <= Xs + Xvec;
				else
					Xvec <= -Xvec;
					
				if(Ys + Yvec > YMIN && Ys + Yvec < YMAX)
					Ys <= Ys + Yvec;
				else
					Yvec <= -Yvec;
			end
			else
			begin
				Xvec <= Xvec + right - left;
				Yvec <= Yvec + up - down;
			end
		      end
		default:
	 	      begin
	 	      	mode <= still;
	 	      	Xvec <= 0;
	 	      	Yvec <= 0;
	 	      end
	endcase
end

endmodule 
